//*****************************************************************************
//*****************************************************************************
{
    PANEL DE MINIATURAS

    Desarrollo: German Estevez y Salvador Jover.
    Fecha: 03/11/2010
    Version: 2010.11.3.2

    Objetivo: Crear un componente que permita cargar una coleccin de imagenes
              tanto en tiempo de ejecucin como de diseo.

              Adems el componente permita no slo cargar la coleccin, sino
              tambin mostrar simultaneamente una lista de miniaturas de dichas
              imgenes almacenadas e interactuar con ellas.

              Existira la posibilidad de realizar la "presentacin" secuencial
              de las imgenes de forma automtica configurando parmetros de tiempo.


    Destino: Publicacion en las respectivas pginas:
                -http://neftali.clubdelphi.com/   y
                -http://www.sjover.com/delphi

    A Revisar: * Problema carga dfm en tiempo de ejecucin para formatos grficos
                 como jpg, png y tif.

               * Diseo e Implementacin de un indice (ItemIndex) que permita
                 identificar la seleccin del usuario. Este ndice se correspondera
                 con el indice de la imagen en la coleccion y tomara valores entre
                 -1 (para no seleccion) y >=0 para una imagen seleccionada.
                 Funcionamiento similar a las propiedades Itemindex de ComboBox y
                 ListBox.
                 Al crearse el panel de miniaturas el indice valdra -1
                 El disparo del evento OnClickPanel situaria el indice en el valor
                 del Item asociado al panel pulsado.
                 Podran existir mtodos como First, Prior, Next y Last, que
                 avanzaran o retrocederan ese indice dentro del dominio de valores
                 que puede tomar el indice de la coleccin [0..Count-1]
                 El panel debera mostrar un marco resaltado si el incide de seleccin
                 coincide con el valor asociado a su indice en la coleccin

               * Mejoras en el refresco de la imagen (parpadeo) con tcnicas de
                 doble buffer.

               * En tiempo de diseo, deberia existir un item con un nombre similar
                 a Paneles de Imagen en el menu contextual del IDE, sobre la clase
                 TPanelMiniaturas.

               * Implementacin de los metodos de carga y manipulacin en tiempo de ejecucin
                 para proximas versiones
                          - LoadFromFile( )     Carga un fichero  (version actual)
                          - LoadFromStream( )   Carga un stream
                          - LoadFromFolder( )   Carga el contenido grfico de una carpeta

                          - SaveToFile ( )      Guarda el contenido de una imagen en un fichero
                          ////- SaveToStream ( )    Guarda el contenido en un stream

                          - Rotaciones
                          - Inversiones
                          etc...
                 * AddImagesFromFolder Slo carga imgenes JPG (pasar a constante)

               * Mejoras posibles:

                    - Simulacion de zoom sobre la miniatura seleccionada
                      para resaltarla
                    - Posibilidad de arrastre desde el explorador de windows
                      y aceptacin si el fichero se corresponde con un formato
                      grfico admitido
                    - Alteracin del orden de los paneles mediante arrastre tactil
                      o de raton, cambiando el orden de la coleccin
                    - Acoplamiento sobre otros objetos
                    - Texturas y mejoras del interfaz (trasparencias, bordes redondeados,
                      etc.)
                    - Poder "pegar" desde el portapapeles de Windows segun el formato
                      de fichero.
                    - Mejorar la insercin de imgenes desde el explorador utilizando
                      el DropPoint.
                    - Permitir reordenar imgenes visualmente (DragDrop).
                    - Modo "parrilla" en lugar de 1 sla lnea de miniaturas.
                    - Posibilidad de aadir mscara a las imagenes (superpuesta
                      mosaico, -marca de agua-, ) 

  // Cambios de la version 2010.11.10
  * Aadido evento OnChange, para cuando cambia el panel que est activo.
  * Integrado el Timer para las rotaciones automticas en el componente. Aade 2
    nuevas propiedades Animate y AnimateInterval, para definir la animacin.
  * Aadida propiedad PanelesCount que devuelve el nmero de elementos de la lista.
  * Aadidos procedimientos DeletePanel y MovePanel (este segundo sn implementar)
    que son los anlogos a AddPanel, para gestionar elementos de la lista de
    miniaturas.
  * AddImagesFromFolder cambia los parmetros que no eran definitivos.
  * Corregido error en Settext (provocaba loop).
  * Aadidas las propiedades AssociatedImage y AssociatedText que corresponden a
    2 componentes que si estn asociados, reflejan los cambios del componentes
    cada vez que se selecciona un elemento de la lista de miniaturas. 


  // Cambios de la versin 2010.11.09
  * Aadida la opcin de aceptar ficheros "arrastrados" desde el explorador.
  * Correcin del mtodo Next (para saltar al primero si ItemIndex = -1)
  * Aadida la opcin de arrastar un directorio adems de arrastrar ficheros.
  * Aadida la propiedad SelectionColor y cambiada la de TipoSeleccion por
    SelectionType (el cambio de nombre es simplemente para que estn juntas en
    el inspector de objetos).


  // Cambios de la versin 2010.11.08
  * Propiedad AutoSlide que permite que el componente de miniaturas se mueva "de forma
    dinmica" para que el panel seleccionado est en la parte visible.
  * Correccin. Liberar memoria en el procedimiento de Seleccin visual de las
    miniaturas.
  * SeleccionaItem y DoClickPanel se ha aadido cdigo para no ejecutar si no ha
    cambiado el ndice.
  * Correccina  la hora de situar un Panel de miniaturas para tener en cuenta el
    desplazamiento actual del Scroll.
  * Activado DoubleBuffer para TPanelMniaturas (create).
  


  // Cambios de la versin 2010.11.5
  * Aadida documentacin al componente.
  * Aadidos mtodos para resaltar la imaegen seleccionada (TPanelImage) segun
    el indice ItemIndex. Y tipo TTipoSeleccion para determinar cmo se quiere hacer.
  * Procedimiento DoClickPanel lo llamamos utilizando en mtodo Click (redefinido) en
    lugar de utilizar el evento Onclick de TPanelImage, ya que de esta forma si se
    asigna en diseo ese evento, se pierde la referencia. (Linea 725 y 630)
  * He camiado la asignacin para utilizar la propiedad en lugar de la variable,
    porque sino el mtodo SetIndexSelection no saltaba nunca. 



 // Cambios en version 2010.11.3.2
 * Aadidas las propiedades en TPanelImagen para el control del foco y de la
   seleccin

         protected
          property IndexSelection: Integer read FIndexSelection write SetIndexSelection;
          property MultipleSelection: Boolean read FMultipleSelection write SetMultipleSelection;

        Para saber un panel que deberia pintar el foco,
        deberia el valor de MultipleSelection ser False y
        el valor de la propiedad IndexSeleccion ser igual
        al valor index en la referencia interna que apunta
        al Item. Es decir

        Pintar resaltado si ->  not MultipleSelection  and IndexSelection = Item.Index

        Si  ((MultipleSelection)  and (IndexSelection = Item.Index))
        estariamos ante una seleccion de multiples paneles para hacer
        algo con ellos: borrarlos por ejemplo. Podria en ese caso existir
        otro tipo de resalte, indicando que es distinto de la seleccion
        para visualizar. Podria pintar en color mas tenue toda la miniatura,
        de forma similar a lo que ocurre cuando en el explorador de ventanas
        cortamos un fichero (el icono modifica el aspecto para indicar que se
        ha cortado y que la operacion no es la de pegado por ejemplo)

 * Aadida propiedad ItemIndex para control de seleccin y los procedimientos
   de navegacion First, Prior, Next y Last sobre dicha propiedad.

 * Corregidas lineas en AddPanel( ) que asignaban el posicionamiento del panel.
   Eran redundates pues la ultima llamada en el procedimient de UpdatePanel( )
   correga la posicin. Dichas lineas sobraban y se han eliminado

 * Aislado GetClientHeightPanel en el calculo del Height del panel

 * Aadidos procedimientos:

      // aadir una imagen desde disco.
    procedure AddImageFromFile(const AFileName: string; const AText:string); virtual;
      // aadir una conjunto de imagenes desde directorio.
    procedure AddImagesFromFolder; virtual;
      // borrar todas las imagenes
    procedure ClearImages;
}



// Cabecera para ayuda generada automticamente.
{:Unit de Definicin de los componentes TPAnelMiniaturas y TPanelImagen.
  Unit de Definicin de los componentes TPAnelMiniaturas y TPanelImagen.
@author Salvador Jover y German Estvez
@cat Package PanelMinuaturas
}
unit UPanelMiniaturas;

//=========================================================================
//
// I N T E R F A C E
//
//=========================================================================
interface

uses
  Messages, SysUtils, Classes, Controls, Graphics, Forms, ExtCtrls, StdCtrls;

const
  KSizeBorderSup = 4;
  KSizeBorderInf = 10;

type

  {: Clase que define el componente para gestionar una lista de miniaturas.
    @Component }
  TPanelMiniaturas = class;
  TMiniaturas = class;
  {: Clase que define un elemento de la lista de miniaturas.}
  TMiniaturaItem = class;

  {: Clase que define el elemento grfico para visualizar un elemento de la
     lista de miniaturas.
    @Component }
  TPanelImagen = class(TCustomControl)
  private
    FItem: TMiniaturaItem;
    FIndexSelection: Integer;
    FMultipleSelection: Boolean;
    FSeparadorPanel : Integer;
    procedure SetImagen(const Value: TPicture);
    procedure SetItem(const Value: TMiniaturaItem);
    procedure SetText(const Value: String);
    function GetImagen: TPicture;
    function GetText: String;
    procedure SetIndexSelection(const Value: Integer);
    procedure SetMultipleSelection(const Value: Boolean);
    procedure SetSeparadorPanel(const Value: Integer);

    //: Permite deseleccionar un elemento visualmente.
    procedure VisualUnselectElement(AIndex:integer);
	  //: Permite seleccionar un elemento visualmente.
    procedure VisualSelectElement(AIndex:integer);
    function GetTPanelMiniaturas: TPanelMiniaturas;
  protected
    //: Nos permite acceder de forma fcil al panel de miniaturas.
    property PanelMiniaturas:TPanelMiniaturas read GetTPanelMiniaturas;
    //: Elemento seleccionado dentro de la coleccin de miniaturas.
    property IndexSelection: Integer read FIndexSelection write SetIndexSelection default -1;
    //: Permite activar la multiseleccin de miniaturas.
    property MultipleSelection: Boolean read FMultipleSelection write SetMultipleSelection default False;

    //: Procedimiento de captura de Click.
    procedure Click; override;

  public
    {: Permite seleccionar el tem.}
    procedure SelecionaItem(const IsMultipleSelection: Boolean; Index: Integer);
    procedure Assign(Source: TPersistent); override;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Paint; override;
    //referencia al item
    property Item: TMiniaturaItem read FItem write SetItem;
    //propiedades obtenidas desde el item
    property Imagen: TPicture read GetImagen write SetImagen;
    {: Propiedad que permite asignar un texto a caa imagen. }
    property Text: String read GetText write SetText;
    {: Define la separacin entre los distintos paneles.}
    property SeparadorPanel: Integer read FSeparadorPanel write SetSeparadorPanel default 0;
  end;

  {: Tipo para el evento OnClickPanel.
     Tipo para el evento OnClickPanel que "salta" cuando el usuario pulsa click
     con el ratn sobre uno de los paneles de imagen.}
  TOnClickPanel = procedure (Sender: TObject; Source: TPanelImagen) of Object;
  {:Tipo para el evento OnChange.
    Tipo para el evento OnChange, que salta cuando cambia el panel activo.}
  TOnChange = TOnClickPanel;
  {: Tipo para el evento OnAddedImage.}
  TOnAddedImage = procedure (AFileName:String) of Object;


  {: Clase que define un elemento de la lista de miniaturas.}
  TMiniaturaItem = class(TCollectionItem)
  private
    FImagenMiniatura: TPicture;
    FText: String;
    procedure SetImagenMiniatura(const Value: TPicture);
    procedure SetText(const Value: String);
  protected
    procedure Update; virtual;
  public
    procedure Assign(Source: TPersistent); override;
    constructor Create(Collection: TCollection); override;
    destructor Destroy; override;
  published
    property ImagenMiniatura: TPicture read FImagenMiniatura write SetImagenMiniatura;
    property Text: String read FText write SetText;
  end;

  TMiniaturas = class(TCollection)
  private
    FContenedor: TPanelMiniaturas;
    function GetItem(Index: Integer): TMiniaturaItem;
    procedure SetItem(Index: Integer; const Value: TMiniaturaItem);
  protected
    function GetOwner: TPersistent; override;
    procedure Update(Item: TCollectionItem); override;
  public
    function AddPanel(Sender: TMiniaturaItem; Index: Integer): TPanelImagen;
    procedure RemovePanel(Sender: TMiniaturaItem);
    constructor Create(AOwner: TPanelMiniaturas);
    destructor Destroy; override;
    function Add: TMiniaturaItem;
    function Insert(Index: Integer): TMiniaturaItem;
    property Items[Index: Integer]: TMiniaturaItem read GetItem write SetItem;
  end;


  {:Tipo para definir cmo "marca" la imagen que est seleccionada de la lista.
    @enum tsNone          Sin seleccin visible.
    @enum tsDebug         Opcin para debug (marca con una 'S').
    @enum tsRectangle     Rectngulo alrededor de la imagen.    
    @enum tsHighlight     Sube la luminosidad de la imagen para resaltarla.
    @enum tsHighRectangle Sube la luminosidad de la imagen y aade un rectngulo.
  }
  TTipoSeleccion = (tsNone, tsDebug, tsRectangle, tsHighlight, tsHighRectangle);

  {: Clase que define el componente para gestionar una lista de miniaturas.}
  TPanelMiniaturas = class(TScrollBox)
  private
    FItemIndex: Integer;
    FPaneles: TList;
    FMiniaturas: TMiniaturas;
    FMiniaturaWidth: Integer;
    FSeparador: Integer;
    FOnClickPanel: TOnClickPanel;
    FSelectionType: TTipoSeleccion;
    FAutoSlide: boolean;
    FSelectionColor: TColor;
    FOnChange: TOnChange;
    AnimTimer:TTimer;
    FAssociatedImage: TImage;
    FAssociatedText: TCustomEdit;
    FOnAddedImage: TOnAddedImage;

    procedure SetMiniaturas(const Value: TMiniaturas);
    function GetPaneles(Index: Integer): TPanelImagen;
    procedure SetMiniaturaWidth(const Value: Integer);
    procedure SetSeparador(const Value: Integer);
    procedure DoClickPanel(Sender: TObject);
    procedure SetOnClickPanel(const Value: TOnClickPanel);
    procedure SetTipoSeleccion(const Value: TTipoSeleccion);
    procedure SetAnimated(const Value: boolean);
    procedure SetAnimateInterval(const Value: cardinal);
    function GetAnimated: boolean;
    function GetAnimateInterval: Cardinal;
  protected
    {: Capura el mensaje de cambio de tamao.}
    procedure WMSize(var Message: TWMSize); message WM_SIZE;
    {: Captura el mensaje de "soltar" imgenes.}
    procedure WMDROPIMAGES(var DropImageMsg: TWMDropFiles); message WM_DROPFILES;
    {: Redefinir el procedimiento Loaded.}
    procedure Loaded; override;
    {: Permite actualizar un panel que visualiza una imagen.}
    procedure UpdatePanels; virtual;
    procedure UpdatePanel(Sender: TMiniaturaItem);
    procedure RemovePanel(Sender: TMiniaturaItem);
    function GetItemIndex: Integer; virtual;
    procedure SetItemIndex(const Value: Integer); virtual;
    function GetClientHeightPanel: Integer;
    //: Permite que el panel ajuste su scroll para visualizar este panel.
    procedure DoAutoSlide(APanel:TPanelImagen);
    {:Permite aadir un nuevo panel a la lista de minuaturas.}
    function AddPanel(APanel:TPanelImagen): Integer;
    {: Procedimiento al que llama el timer cuando se activa la animacin.}
    procedure DoAnimation(Sender: TObject); virtual;
    
  public
    {: Redefinido el constructor de la clase para asignar valores iniciales.}
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    {: Devuele el nmero de miniaturas del panel.}
    function Count(): Integer;
    {: Mtodo que permite eliminar una miniatura de la lista.}
    procedure DeletePanel(AIndex:integer);
    {: Mtodo que permite mover una miniatura dentro del componente.}
    procedure MovePanel(APanel:TPanelImagen; ANewIndex:Integer);
    function IndexOf(Item: TMiniaturaItem): Integer;
    {: Procedimiento para marcar el primer elemento como activo.}
    procedure First;
    {: Procedimiento para marcar el anterior elemento como activo.}
    procedure Prior;
    {: Procedimiento para marcar el siguiente elemento como activo.}
    procedure Next;
    {: Procedimiento para marcar el ltimo elemento como activo.}
    procedure Last;
    //: Paneles definidos en el componente.
    property Paneles[Index: Integer]: TPanelImagen read GetPaneles;
    {: Aadir una imagen desde disco.}
    procedure AddImageFromFile(const AFileName: string; const AText:string); virtual;
    {: Permite aadir todas las imgenes de una carpeta al componente.}
    procedure AddImagesFromFolder(const AFolderName:string); virtual;
    {: Eliminar todas las miniaturas existentes en el componente.}
    procedure ClearImages;

  published

    {: Define el ancho del "espacio" para la visualizacin de cada miniatura. }
    property MiniaturaWidth: Integer read FMiniaturaWidth write SetMiniaturaWidth default 50;
    {: Espacio definido entre las diferentes minuaturas definidas en la lista.}
    property Separador: Integer read FSeparador write SetSeparador default 10;
    {: Acceso a las miniaturas creadas en la coleccin.}
    property Miniaturas: TMiniaturas read FMiniaturas write SetMiniaturas;
    {: Indica el ndice de la miniatura selecciinada en cada momento.}
    property ItemIndex: Integer read GetItemIndex write SetItemIndex default -1;
    {: Indica si el control debe desplazar las imgenes automticamente segun la
      que hay seleccionada en cada momento.}
    property AutoSlide:boolean read FAutoSlide write FAutoSlide;
    {: Indica la forma en que visualmente se "marca" la imagen activa. }
    property SelectionType:TTipoSeleccion read FSelectionType write SetTipoSeleccion;
    {: Indica el color que se aplica al rectnguo de seleccion (si se aplica).}
    property SelectionColor:TColor read FSelectionColor write FSelectionColor;
    {:Permite definir si el componente hace "pase de diapositivas" con las imgenes que contiene.
     @seealso <See Property="AnimateInterval"> velocidad de animacin (ms).}
    property Animated:boolean read GetAnimated write SetAnimated;
    {: Define el intervalo en milisegundos para la animacin de las imgenes.
      @seealso <See Property="Animated"> Define si se activa o no la animacin.}
    property AnimateInterval:Cardinal read GetAnimateInterval write SetAnimateInterval;
    {: Componente derivado de TImage que visualiza la imagen asociada a la miniatura seleccionada.
      @seealso <See Property="AssociatedText">.}
    property AssociatedImage:TImage read FAssociatedImage write FAssociatedImage;
    {: Componente derivado de TCustomEdit que visualiza el texto asociado a la miniatura seleccionada.
      @seealso <See Property="AssociatedImage">.}
    property AssociatedText:TCustomEdit read FAssociatedText write FAssociatedText;

    {: Evento OnClickPanel
       Evento OnClickPanel que "salta" cuando el usuario pulsa click con el
       ratn sobre uno de los paneles de imagen.
      @seealso <See Event="OnChange">.
      @Event }
    property OnClickPanel: TOnClickPanel read FOnClickPanel write SetOnClickPanel;
    {: Evento OnChange para detectar cuando cambia el elemento seleccionado.
    @seealso <See Event="OnClickPanel">.
    @Event}
    property OnChange:TOnChange read FOnChange write FOnChange;
    {: Evento OnAddedImage que "salta" cuando se aade una imagen a a coleccin.
      @Event }
    property OnAddedImage:TOnAddedImage read FOnAddedImage write FOnAddedImage;
  end;


//: Devuelve una lista de ficheros.
procedure FindFiles(StartDir, FileMask: string; recursively: boolean;
                    var FilesList: TStringList);


//=========================================================================
//
// I M P L E M E N T A T I O N
//
//=========================================================================
implementation

uses
  ShellAPI, Math,
  Windows, Types, Dialogs, FileCtrl, GraphUtil;


{: Devuelve una lista de nombres de fichero encontrados a partir de la carpeta
   inicial StartDir, que cumplen el patrn especificado por FileMask. Mediante
   recursively se indica si se desea hacer la busqueda en los subdirectorios.
  @param    StartDir    Carpeta desde la que empezar a buscar.
  @param    FileMask    Patrn que han de cumplir los ficheros.
  @param    recursively Si hay que continuar la bsuqueda en los subdirectorios.
  @param    FilesList   Lista con los nombres de fichero encontrados.)
@example  Buscar todos los ficheros de tipo JPG del directorio y subcarpetas:<br>
<Code>    
	path:string; 
	fileList:TStringList;
begin 
  ... 
  path := 'c:\media\images\'; 
  FindFiles(path, '*.jpg', True, fileList);
</Code>
}
procedure FindFiles(StartDir, FileMask: string; recursively: boolean; var FilesList: TStringList);
var
  SR: TSearchRec;
  DirList: TStringList;
  IsFound: Boolean;
  i: integer;
begin

  if (StartDir[length(StartDir)] <> '\') then begin
    StartDir := StartDir + '\';
  end;

  // Crear la lista de ficheos en el directorio StartDir (no directorios!)
  IsFound := FindFirst(StartDir + FileMask, faAnyFile - faDirectory, SR) = 0;
  // MIentras encuentre
  while IsFound do  begin
    FilesList.Add(StartDir + SR.Name);
    IsFound := FindNext(SR) = 0;
  end;

  SysUtils.FindClose(SR);

  // Recursivo?
  if (recursively) then begin
    // Build a list of subdirectories
    DirList := TStringList.Create;
    // proteccion
    try
      IsFound := FindFirst(StartDir + '*.*', faAnyFile, SR) = 0;
      while IsFound do
      begin
        if ((SR.Attr and faDirectory) <> 0) and
          (SR.Name[1] <> '.') then
          DirList.Add(StartDir + SR.Name);
        IsFound := FindNext(SR) = 0;
      end;
      SysUtils.FindClose(SR);

      // Scan the list of subdirectories
      for i := 0 to DirList.Count - 1 do
        FindFiles(DirList[i], FileMask, recursively, FilesList);
    finally
      DirList.Free;
    end;
  end;
end;


{ TPanelImagen }
procedure TPanelImagen.Assign(Source: TPersistent);
begin
  if Source is TPanelImagen then
  begin
    Item:= TPanelImagen(Source).Item;
    FMultipleSelection:= False;
    FIndexSelection:= -1;
  end
  else inherited;

end;

//: redefinicin del constructor del componente.
constructor TPanelImagen.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  Height:= 105;
  Width:= 105;
  FSeparadorPanel:= 0;
  FMultipleSelection:= False;
  FIndexSelection:= -1;
end;

destructor TPanelImagen.Destroy;
begin
  FItem:= nil;
  inherited Destroy;
end;

function TPanelImagen.GetImagen: TPicture;
begin
  if Assigned(Item) then
    Result:= Item.FImagenMiniatura
  else Result:= Nil;
end;

function TPanelImagen.GetText: String;
begin
  if Assigned(Item) then
     Result:= Item.Text
  else Result:= '';
end;

{:Permite deseleccionar visualmente el elementos activo de la llista.
  seealso <See Method="VisualSelectElement"> Selecccionar/resaltar un elemento .}
procedure TPanelImagen.VisualUnselectElement(AIndex: integer);
begin
  PanelMiniaturas.Paneles[AIndex].Invalidate;
end;

{:Permite seleccionar visualmente el elementos activo de la llista.
  Permite seleccionar visualmente el elementos que est seleccionado en la lista
  con la propiedad IndexSelection.
  seealso <See Property="IndexSelection"> Elemento seleccionado.
  seealso <See Method="VisualUnselectElement"> Deselecccionar un elemento.}
procedure TPanelImagen.VisualSelectElement(AIndex: integer);
var
  rect, rectD:TRect;
  oldw:integer;
  oldColor:TColor;
  oldS:TBrushStyle;
  bm:TImage;
  //----------------------------------------------------------------------------
  procedure AdjustLuma(ACanvas:TCanvas);
  var
    i,j:Integer;
    col:TColor;
  begin
    // Al cambiar
    for i := 0 to (ACanvas.ClipRect.Right) do begin
      for j := 0 to (ACanvas.ClipRect.Bottom)  do begin
        col := ACanvas.Pixels[i,j];
        col := ColorAdjustLuma(col, 70, True);
        ACanvas.Pixels[i,j] := col;
      end;
    end;
  //----------------------------------------------------------------------------
end;
begin

  // No hay seleccin de imagen? ==> Salimos
  if (PanelMiniaturas.SelectionType = tsNone) then begin
    Exit;
  end;

  // Valores originales
  oldColor := Canvas.Pen.Color;
  oldw := Canvas.Pen.Width;
  oldS := Canvas.Brush.Style;

  // proteccion
  try
    // Seleccin del elemento
    Canvas.Pen.Color := PanelMiniaturas.SelectionColor;
    Canvas.Pen.Width := 3;
    Canvas.Brush.Style := bsClear;

    // rectngulo?
    if (PanelMiniaturas.SelectionType = tsRectangle) or
       (PanelMiniaturas.SelectionType = tsHighRectangle) then begin

      // Calcular medidas
      Rect := Self.ClientRect;
      InflateRect(Rect, -1, -1);
      // pintado
      Canvas.Rectangle(Rect);
    end;

    // Hay que aplicar subir la luminosidad?
    if (PanelMiniaturas.SelectionType = tsHighlight) or
       (PanelMiniaturas.SelectionType = tsHighRectangle) then begin

      // Crear la imagen "resaltada"
      bm := TImage.Create(nil);
      try
        // Copiamos el canvas
      bm.Picture.Bitmap.Height := Canvas.ClipRect.Bottom - Canvas.ClipRect.Top - 2;
      bm.Picture.Bitmap.Width := Canvas.ClipRect.Right - Canvas.ClipRect.Left - 2;
      Rect := Canvas.ClipRect;
      InflateRect(Rect, -2, -2);
      RectD := bm.Picture.Bitmap.Canvas.ClipRect;
      bm.Picture.Bitmap.Canvas.CopyRect(RectD, Canvas, Rect);
        // Ajustar luminosidad
      AdjustLuma(bm.Picture.Bitmap.Canvas);
        // Repintar
      Canvas.StretchDraw(Rect, bm.Picture.Bitmap);
      finally
        FreeAndNil(bm);
      end;
    end;

    // Modo Debug? (me daba pena borrarlo  ;-)  )
    if (PanelMiniaturas.SelectionType = tsDebug) then begin
      Canvas.Brush.Style := bsSolid;
      Canvas.TextOut(0, 0, 'S'); //BOTON SELECCIONADO
    end;

  finally
    Canvas.Pen.Color := oldColor;
    Canvas.Pen.Width := oldw;
    Canvas.Brush.Style := oldS;
  end;
end;


{: Mtodo redefinido para el pintado del panel.
   Mtodo redefinido para el pintado del panel. Se tendr en cuenta el elemento
   activo en cada meomento para la lista de imgenes; En este casi si el elemento
   que se pinta es el activo o no.
   seealso <See Method="VisualUnselectElement"> Deselecccionar un elemento.
   seealso <See Method="VisualSelectElement"> Selecccionar/resaltar un elemento .}   
procedure TPanelImagen.Paint();
var
 Rect: TRect;
begin

  //DEBUG para pintado...
  //OutputDebugString(PChar(Format('+Paint   Item.Index:%d   IndexSelection:%d',[Item.Index,IndexSelection])));

  inherited Paint;

  if Assigned(Imagen) then begin
    Rect.Left:= 2;
    Rect.Top:= 2;
    Rect.Bottom:= Height-2;
    Rect.Right:= Width-2;
    Canvas.Rectangle(1,1, ClientWidth-1, ClientHeight-1);
    
    Canvas.StretchDraw(Rect, Imagen.Graphic);

    //Seleccionar visualmente
    if (Item.Index = self.IndexSelection) and (not FMultipleSelection) then
    begin
      VisualSelectElement(Self.IndexSelection);
    end;
  end;

end;

procedure TPanelImagen.SelecionaItem(const IsMultipleSelection: Boolean; Index: Integer);
begin

  // No ha cambiado?
  if (IndexSelection = Index) then begin
    Exit;
  end;

  IndexSelection:= Index;
  MultipleSelection:= IsMultipleSelection;

  // desplazar el scroll si es necesario
  if (PanelMiniaturas.AutoSlide) then begin
    PanelMiniaturas.DoAutoSlide(Self);
  end;

  Invalidate;
end;

procedure TPanelImagen.SetImagen(const Value: TPicture);
begin
  Imagen.Assign(Value);
end;

procedure TPanelImagen.SetIndexSelection(const Value: Integer);
begin
  // Ha cambiado?
  if (FIndexSelection <> -1) and (FIndexSelection <> Value) then begin
    // Antes de asignar el nuevo "limpiamos" el antiguo
    VisualUnselectElement(FIndexSelection);
  end;
  //Asignar
  FIndexSelection := Value;
end;

procedure TPanelImagen.SetItem(const Value: TMiniaturaItem);
begin
  if Value <> FItem then FItem := Value;
end;

procedure TPanelImagen.SetMultipleSelection(const Value: Boolean);
begin
  FMultipleSelection := Value;
end;

procedure TPanelImagen.SetSeparadorPanel(const Value: Integer);
begin
  if Value <> FSeparadorPanel then begin
    FSeparadorPanel := Value;
    //aqui no ejecutamos update panels sino update panel individual
    Item.Update;
  end;

end;

procedure TPanelImagen.SetText(const Value: String);
begin
  if Text <> Value then begin
    Item.Text := Value;
  end;
end;

{:Procedimiento que redefine el Click para capturar la accin.}
procedure TPanelImagen.Click();
begin
  inherited;
  // Asignado? (proteccion)
  if Assigned(FItem.Collection) then begin
    TMiniaturas(FItem.Collection).FContenedor.DoClickPanel(Self);
  end;
end;

function TPanelImagen.GetTPanelMiniaturas: TPanelMiniaturas;
begin
  Result := TPanelMiniaturas(Self.Owner);
end;

{ TMiniaturaItem }

procedure TMiniaturaItem.Assign(Source: TPersistent);
begin
  if Source is TMiniaturaItem then
  begin
    FImagenMiniatura.Assign(TMiniaturaItem(Source).ImagenMiniatura);
    FText:= TMiniaturaItem(Source).Text;
  end else inherited Assign(Source);
end;

constructor TMiniaturaItem.Create(Collection: TCollection);
begin
  FImagenMiniatura:= TPicture.Create;
  inherited Create(Collection);
  TMiniaturas(Collection).AddPanel(Self, Index);
end;

destructor TMiniaturaItem.Destroy;
begin
  //eliminamos el panel asociado a la  miniatura
  TMiniaturas(Collection).RemovePanel(Self);
  //destruimos la imagen
  FImagenMiniatura.Free;
  inherited Destroy;
end;

procedure TMiniaturaItem.SetImagenMiniatura(const Value: TPicture);
begin
  FImagenMiniatura.Assign(Value);
  Update;
end;


procedure TMiniaturaItem.SetText(const Value: String);
begin
  if FText <> Value then FText := Value;
end;

procedure TMiniaturaItem.Update;
begin
  TMiniaturas(Collection).Update(Self);
end;

{ TMiniaturas }

constructor TMiniaturas.Create(AOwner: TPanelMiniaturas);
begin
  inherited Create(TMiniaturaItem);
  FContenedor:= AOwner;
end;

function TMiniaturas.Add: TMiniaturaItem;
begin
  Result:= TMiniaturaItem(inherited Add);
  if Assigned(FContenedor) then FContenedor.UpdatePanel(Result);
end;

function TMiniaturas.GetItem(Index: Integer): TMiniaturaItem;
begin
  Result:= TMiniaturaItem(inherited GetItem(Index));
end;

function TMiniaturas.GetOwner: TPersistent;
begin
  Result:= FContenedor;
end;

procedure TMiniaturas.SetItem(Index: Integer; const Value: TMiniaturaItem);
begin
  inherited SetItem(Index, Value);
end;

procedure TMiniaturas.Update(Item: TCollectionItem);
begin
 if Assigned(FContenedor) then
   if Assigned(Item) then FContenedor.UpdatePanel(Item as TMiniaturaItem)
   else FContenedor.UpdatePanels;
end;

function TMiniaturas.AddPanel(Sender: TMiniaturaItem; Index: Integer): TPanelImagen;
begin
  Result:= TPanelImagen.Create(Self.FContenedor);

  // -G- Result.OnClick:= FContenedor.DoClickPanel;
  // -G- (05/11/2010) Pasa al AddPanel porque de esta forma perdemos el evento
  // -G- OnClick y si en dieso se asigna, esta referencia se pierde.
  Result.Parent:= FContenedor;
  Result.Item:= Sender;  //ligadura al item
  FContenedor.AddPanel(Result);
  FContenedor.UpdatePanel(Sender);
end;

function TMiniaturas.Insert(Index: Integer): TMiniaturaItem;
begin
  Result:= TMiniaturaItem(inherited Insert(Index));
 if Assigned(FContenedor) then FContenedor.UpdatePanels;
end;

procedure TMiniaturas.RemovePanel(Sender: TMiniaturaItem);
begin
  //la accion se ejecuta tambien cuando se esta destruyendo el componente
  //principal por lo que evitamos que se lanze el metodo apuntando a nil
  //comprobando antes que FContenedor es vlido
  if Assigned(FContenedor) then FContenedor.RemovePanel(Sender);
end;

destructor TMiniaturas.Destroy;
begin
  FContenedor:= Nil;
  inherited Destroy;
end;


{ TPanelMiniaturas }

procedure TPanelMiniaturas.Loaded();
begin
  inherited Loaded;
  DragAcceptFiles(Handle, True);
end;

{:Permite capturar imgenes arrastradas desde el exploador.
  Permite capturar imgenes arrastradas desde el exploador. Se puede arrastrar una
  o varias imgenes seleccionadas o una carpeta completa.
}
procedure TPanelMiniaturas.WMDROPIMAGES(var DropImageMsg: TWMDropFiles);
var
  fImagesCount: Integer;
  fFileName: array[0..MAX_PATH] of Char;
  //fDropPoint: TPoint;
  i: Integer;
begin
  //numero de ficheros arrastrados
  fImagesCount:= DragQueryFile(DropImageMsg.Drop, $FFFFFFFF, nil, 0);
  //obtenemos el punto en el que es soltado el conjunto de ficheros
  //DragQueryPoint(DropImageMsg.Drop, fDropPoint);
  try
    for i := 0 to fImagesCount - 1 do
    begin
       DragQueryFile(DropImageMsg.Drop, i, fFilename, MAX_PATH);

      // Hay uno Slo? (por si es una carpeta)
      if (fImagesCount = 1) then begin
        // Directorio? ==> Entonces arrastramos todas las imgenes...
        if (DirectoryExists(fFilename)) then begin
          // Aadir todo el directorio
          AddImagesFromFolder(fFilename);
          Break;
        end;
      end;
      // Aadir el fichero individual
      AddImageFromFile(String(fFilename), '');
    end;
  finally
    DragFinish(DropImageMsg.Drop);
  end;
end;

{: Redefinicin del constructor del componente.}
constructor TPanelMiniaturas.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FPaneles:= TList.Create;
  // Timer de la animacin
  AnimTimer := TTimer.Create(Self);
  AnimTimer.OnTimer := DoAnimation;
  AnimTimer.Enabled := False;
  AnimTimer.Interval := 750;
  FMiniaturas:= TMiniaturas.Create(self);
  // Otras inicializaciones
  Height:= 105;
  Width:= 300;
  Constraints.MinHeight:= 90;
  Constraints.MinWidth:= 65;
  VertScrollBar.Visible:= False;
  FMiniaturaWidth:= 50;
  FSeparador:= 10;
  FItemIndex:= -1;
  FAutoSlide := true;
  DoubleBuffered := True;
  FSelectionType := tsRectangle;
  FSelectionColor := clLime;

end;

destructor TPanelMiniaturas.Destroy;
var
 i: Integer;
begin
  FMiniaturas.Free;
  for i := 0 to FPaneles.Count - 1 do
  begin
   TPanelImagen(FPaneles[i]).Free;
  end;
  FPaneles.Clear;
  FPaneles.Free;
  AnimTimer.Enabled := False;
  AnimTimer.Free;
  inherited Destroy;
end;

procedure TPanelMiniaturas.SetMiniaturaWidth(const Value: Integer);
begin
  if Value <> FMiniaturaWidth then
  begin
    FMiniaturaWidth := Value;
    UpdatePanels;
  end;
end;

procedure TPanelMiniaturas.SetOnClickPanel(const Value: TOnClickPanel);
begin
  FOnClickPanel := Value;
end;

procedure TPanelMiniaturas.SetSeparador(const Value: Integer);
begin
  if Value <> FSeparador then
  begin
    FSeparador := Value;
    UpdatePanels;
  end;
end;

{: Activar el elemento activo de la coleccin.}
procedure TPanelMiniaturas.SetItemIndex(const Value: Integer);
var
 oldValue: Integer;
begin
  if (Value >= Count) or (Value < -1)   then
    raise Exception.Create('Error indice fuera de rango');

   if (Value <> FItemIndex) and (Count > 0) then
   begin
      oldValue:= FItemIndex;
      FItemIndex := Value;
      if (oldValue >= 0) and (oldValue < Count) then
         TPanelImagen(FPaneles[oldValue]).SelecionaItem(False, -1);
      if Value >= 0 then
         TPanelImagen(FPaneles[Value]).SelecionaItem(False, Value);
   end
   else if Count = 0 then FItemIndex:= -1;


  // hay control asociado de imagen?
  if Assigned(FAssociatedImage) then begin
    if (Value = -1) then begin
      FAssociatedImage.Picture.Assign(nil);
    end
    else begin
      FAssociatedImage.Picture.Assign(TPanelImagen(FPaneles[Value]).Imagen);
    end;
  end;

  // Hay control asociado de texto?
  if Assigned(FAssociatedText) then begin
    if (Value = -1) then begin
      FAssociatedText.Text := '';
    end
    else begin
      FAssociatedText.Text := TPanelImagen(FPaneles[Value]).Text;
    end;
  end;

   // Lanzar el evento de Cambio
   if Assigned(Self.FOnChange) then begin
     Self.OnChange(Self, Paneles[FItemIndex]);
   end;

end;

procedure TPanelMiniaturas.SetMiniaturas(const Value: TMiniaturas);
begin
  FMiniaturas.Assign(Value);
end;

{
  PENDIENTE:
     Temporalmente la altura del panel se calcula con un calculo de 3/4 de
     la altura del panel de miniaturas.
     Para hacerlo correctamente, deberiamos obtener el tamao de la barra
     de desplazamiento y descontarla.
     Este calculo afecta tanto a la funcion UpdatePanel( ) como a UpdatePanels por
     lo que se podra encapsular en una funcion y llamarla desde los dos puntos
     de asignacin.
}

procedure TPanelMiniaturas.UpdatePanel(Sender: TMiniaturaItem);
var
  i: Integer;
  pnl: TPanelImagen;
begin
   i:= IndexOf(Sender);
   if i <> -1 then
   begin
     pnl:= FPaneles[i];
     pnl.Top := KSizeBorderSup;;
     pnl.Left:= (Miniaturas.Items[i].Index * MiniaturaWidth) + ((i+ 1) * Separador) + pnl.SeparadorPanel -
                {Correccion del HScroll} Self.HorzScrollBar.ScrollPos;

     //DEBUG para update
     OutputDebugString(PChar(Format('ClientRect: (%d,%d)-(%d,%d)',
      [ClientRect.Left, ClientRect.Top, ClientRect.Right, ClientRect.Bottom])));
     OutputDebugString(PChar(Format('Horiz. Scroll;  ScrollPos= %d     Position=%d',
      [HorzScrollBar.ScrollPos, HorzScrollBar.Position])));


     pnl.Height:= GetClientHeightPanel;
     pnl.Width:= MiniaturaWidth;
     pnl.Invalidate;
   end;
end;

{:Permite actualizar los paneles que visualizan las imagenes.
  Permite actualizar los paneles que visualizan las imagenes.
  @seealso <See Method="UpdatePanel"> Mtodo UpdatePanel para actualizar 1 panel.
  @seealso <See Method="RemovePanel"> Mtodo RemovePanel para eliminar 1 panel.
 }
procedure TPanelMiniaturas.UpdatePanels();
var
  i,j: Integer;
  pnl: TPanelImagen;
begin
   for i := 0 to Miniaturas.Count - 1 do
   begin
     j:= IndexOf(Miniaturas.Items[i]);
     if j <> -1 then
     begin
       pnl:= FPaneles[j];
       pnl.Top := KSizeBorderSup;;
       pnl.Left:= (Miniaturas.Items[i].Index * MiniaturaWidth) + ((i+ 1) * Separador) + pnl.SeparadorPanel -
                {Correccion del HScroll} Self.HorzScrollBar.ScrollPos;
       pnl.Height:= GetClientHeightPanel;
       pnl.Width:= MiniaturaWidth;
       pnl.Invalidate;
     end;
   end;
end;


procedure TPanelMiniaturas.WMSize(var Message: TWMSize);
begin
  inherited;
  UpdatePanels;
end;


{:Aadir una imagen desde disco.
  @param    AFileName   Path completo del fichero que contiene la imagen.
  @param    AText       Texto asociado a la imagen a aadir.
  @seealso <See Method="AddImagesFromFolder">
  @example  Aadir imgenes desde un fichero de disco:<br>
<Code>
var
  path:string;
begin
  ...
  path := 'c:\media\images\foto1.jpg';
  panelMini.AddImageFromFile(path, 'fotografa1');
</Code>
}
procedure TPanelMiniaturas.AddImageFromFile(const AFileName, AText: string);
var
  item:TMiniaturaItem;
begin
  // Crear el item y aadirlo
  item := TMiniaturaItem.Create(Self.Miniaturas);
  item.ImagenMiniatura.LoadFromFile(AFileName);
  item.Text := AText;

  // Asignado el evento?
  if Assigned(Self.FOnAddedImage) then begin
    Self.FOnAddedImage(AFileName);
  end;

end;

{:Mtodo que permite eliminar una miniatura de la lista.
  Mtodo que permite eliminar una miniatura de la lista. Recibe como parmetro
  el ndice del elemento a eliminar.
  @param AIndex  Indice del elemento (miniatura) que se va a eliminar.
  @seealso <See Method="MovePanel"> Mtodo MovePanel para mover una minioatura.
  @example  Buscar todos los ficheros de tipo JPG del directorio y subcarpetas:<br>
<Code>
procedure TForm1.Button2Click(Sender: TObject);
begin
  // Nada seleccionado
  if (pnlMiniaturas.ItemIndex = -1) then begin
    MessageDlg('No hay ningun elemento seleccionado. ',
               mtWarning, [mbOK], 0);
    Exit;
  end;
  // Eliminar el panel seleccionado
  pnlMiniaturas.DeletePanel(pnlMiniaturas.ItemIndex);
end;
</Code>
}
procedure TPanelMiniaturas.DeletePanel(AIndex: integer);
var
  pnl:TPanelImagen;
begin

  // Eliminar ese elemento
  pnl := Paneles[AIndex];
  pnl.Item.Free;

  ItemIndex := Min(ItemIndex, Count-1);

  // Hay Index correcto?
  if (ItemIndex <> -1) then begin
    // Actualizar el que queda como actual
    Paneles[ItemIndex].IndexSelection := ItemIndex;
    Paneles[ItemIndex].Paint;
  end;
end;


{:Mtodo que permite mover una miniatura dentro del componente.
  Mtodo que permite mover una miniatura dentro del componente. Recibe como parmetro
  el ndice el elemento a mover y el nuevo ndice.
  @param APanel     Elemento que se desea mover dentro de la lista.
  @param ANewIndex  Indice de la nueva posicin en la lista.
  @seealso <See Method="DeletePanel"> Mtodo DeletePanel para eliminiar una miniatura.
  @example  Buscar todos los ficheros de tipo JPG del directorio y subcarpetas:<br>
<Code>
procedure TForm1.Button2Click(Sender: TObject);
begin
  // Mover el panel selecconado a la posicin 6
  pnlMiniaturas.MovePanel(pnlMiniaturas.Paneles[lbElementos.ItemIndex], 6);
end;
</Code>
}
procedure TPanelMiniaturas.MovePanel(APanel: TPanelImagen; ANewIndex: Integer);
var
  i:Integer;
begin

  i := APanel.Item.Index;

  // es la misma posicion? => Nada
  if (i = ANewIndex) then begin
    // Nada
  end
  else if (i < ANewIndex) then begin

    // proxima version

  end
  else if (i > (Count - 1)) then begin

    // proxima version

  end
  else if (i > ANewIndex) then begin

    // proxima version

  end;
end;


{:Permite aadir todas las imgenes de una carpeta al componente.
  Permite aadir todas las imgenes de una carpeta al componente. Se hace de
  forma recursiva, asi que tambin se aadirn si dentro de la carpeta existen
  otras.
  @param    AFolderName   Path de la careta que contiene las imgenes.
  @seealso <See Method="AddImageFromFile">
  @example  Aadir imgenes desde una carpeta:<br>
<Code>
var
  path:string;
begin
  ...
  // Si se selecciona un nuevo directorio ...
  if (SelectDirectory('Seleccionar directorio...', '', path)) then begin
    // cargar las imgenes de la carpeta
    pnlMiniaturas.AddImagesFromFolder(path);
  end;
</Code>
}
procedure TPanelMiniaturas.AddImagesFromFolder(const AFolderName:string);
var
  imList: TStringList;
  i:Integer;
begin
  // Limpiar las listas
  imList:= TStringList.Create;
  try
    // cargar las imgenes (slo JPG)
    FindFiles(AFolderName, '*.jpg', True, imList);
    FindFiles(AFolderName, '*.bmp', True, imList);
    FindFiles(AFolderName, '*.wmf', True, imList);
    FindFiles(AFolderName, '*.emf', True, imList);
    // Aadirlas
    for i := 0 to (imList.Count - 1) do  begin
      AddImageFromFile(imList[i], '...');
    end;
  finally
    FreeAndNil(imList);
  end;
end;

{:Permite aadir un nuevo panel a la lista de minuaturas.
  @param  TPanelImagen  Nuevo panel a aadir.
  @seealso clase TPanelImagen: <see class="TPanelImagen">}
function TPanelMiniaturas.AddPanel(APanel:TPanelImagen): Integer;
begin
  Result:= FPaneles.Add(APanel);
end;

{:Eliminar todas las miniaturas existentes en el componente.
  Eliminar todas las miniaturas existentes en el componente. El ndice una vez
  eliminadas (ItemIndex) vuelve a quedarse a -1.}
procedure TPanelMiniaturas.ClearImages;
var
  i: Integer;
begin
  //la llamada al borrado de la imagen destruye cada panel.
  for i := Count-1 downto 0 do
  begin
    Miniaturas.Items[i].Free;
  end;
end;

{:Devuele el nmero de miniaturas del panel.
  Devuele el nmero de miniaturas aadidas al panel.
  @result  Integer 	Nmero de elementos en la lista.}
function TPanelMiniaturas.Count: Integer;
begin
  Result:= FMiniaturas.Count;
end;

{:Devuele la altura (height) del panel de miniaturas.
  Devuele el valor del heigh del panel, de acuerdo al area cliente del contenedor.
  @result  Integer 	}
function TPanelMiniaturas.GetClientHeightPanel: Integer;
begin
  Result:= (Height - KSizeBorderSup - KSizeBorderInf - GetSystemMetrics(SM_CYHSCROLL));
end;

function TPanelMiniaturas.GetItemIndex: Integer;
begin
  Result := FItemIndex;
end;

function TPanelMiniaturas.GetPaneles(Index: Integer): TPanelImagen;
begin
  if (Index < 0) or (Index >= FPaneles.Count) then
  Result:= nil
  else Result:= FPaneles[Index];
end;


function TPanelMiniaturas.IndexOf(Item: TMiniaturaItem): Integer;
var
 i: Integer;
begin
  Result:= -1;
  for i := 0 to FPaneles.Count - 1 do
  begin
    if TPanelImagen(FPaneles[i]).Item = Item then
    begin
       Result:= i;
       Exit;
    end;
  end;
end;

{: Procedimiento para marcar el primer elemento como activo.
@seealso <See Method=Last> Activar el ltimo elemento de la lista de miniaturas.
@seealso <See Method=Next> Activar el siguiente elemento de la lista de miniaturas.
@seealso <See Method=Prior> Activar el anterior elemento de la lista de miniaturas.
}
procedure TPanelMiniaturas.First;
begin
  if Count = 0 then Exit;
  ItemIndex:= 0;
end;

{: Procedimiento para marcar el ltimo elemento como activo.
@seealso <See Method=First> Activar el primer elemento de la lista de miniaturas.
@seealso <See Method=Next> Activar el siguiente elemento de la lista de miniaturas.
@seealso <See Method=Prior> Activar el anterior elemento de la lista de miniaturas.
}
procedure TPanelMiniaturas.Last;
begin
  if Count = 0 then Exit;
  ItemIndex:= Count -1;
end;

{: Procedimiento para marcar el siguiente elemento como activo.
@seealso <See Method=First> Activar el primer elemento de la lista de miniaturas.
@seealso <See Method=Last> Activar el ltimo elemento de la lista de miniaturas.
@seealso <See Method=Prior> Activar el anterior elemento de la lista de miniaturas.
}
procedure TPanelMiniaturas.Next;
var
 i: Integer;
begin
  if Count = 0 then Exit;
  if ItemIndex <> -1 then
  begin
    i:= ItemIndex + 1;
    if i <= Count - 1 then ItemIndex:= i;
  end
  else First; //si es -1 vamos al primero
  end;

{: Procedimiento para marcar el anterior elemento como activo.
@seealso <See Method=Last> Activar el ltimo elemento de la lista de miniaturas.
@seealso <See Method=Next> Activar el siguiente elemento de la lista de miniaturas.
@seealso <See Method=First> Activar el primer elemento de la lista de miniaturas.
}
procedure TPanelMiniaturas.Prior;
var
 i: Integer;
begin
  if Count = 0 then Exit;
  if ItemIndex <> -1 then
  begin
    i:= ItemIndex - 1;
    if i >= 0 then ItemIndex:= i
  end
  else First;  //si es -1 vamos al primero
end;

procedure TPanelMiniaturas.RemovePanel(Sender: TMiniaturaItem);
var
  i: Integer;
begin
   i:= IndexOf(Sender);
   if i <> -1 then
   begin
     TPanelImagen(FPaneles[i]).Free;
     FPaneles.Delete(i);
     UpdatePanels;
   end;
end;


procedure TPanelMiniaturas.DoClickPanel(Sender: TObject);
var
  old_ItemIndex: Integer;
begin
  old_ItemIndex:= FItemIndex;

  // No ha cambiado?
  if (FItemIndex = TPanelImagen(Sender).FItem.Index) then begin
    Exit;
  end;

  Self.ItemIndex:= TPanelImagen(Sender).FItem.Index;
  if (old_ItemIndex <> FItemIndex) and (old_ItemIndex <> -1)  then begin
   TPanelImagen(FPaneles[old_ItemIndex]).SelecionaItem(False, -1);
  end;

  TPanelImagen(Sender).SelecionaItem(False, FItemIndex);

  if Assigned(FAssociatedImage) then begin
    FAssociatedImage.Picture.Assign(TPanelImagen(Sender).Imagen);
  end;

  if Assigned(FOnClickPanel) then begin
    FOnClickPanel(Self, TPanelImagen(Sender));
  end;
end;




procedure TPanelMiniaturas.SetTipoSeleccion(const Value: TTipoSeleccion);
begin
  FSelectionType := Value;
  if (ItemIndex <> -1) then begin
    Miniaturas.Items[ItemIndex].Update;
  end;
end;

{:Permite que el panel ajuste su scroll para visualizar este panel.
  Permite que el panel ajuste su scroll para visualizar este panel.
  @param    APanel    Panel seleccionado que se desea que est visible.
  @seealso clase TPanelImagen: <see class="TPanelImagen">
  }
procedure TPanelMiniaturas.DoAutoSlide(APanel: TPanelImagen);
var
  i:Integer;
  newpos:integer;
begin

  // Buscar en panel
  i := FPaneles.IndexOf(APanel);
  // encontrado?
  if (i > -1) then begin

    newpos := (APanel.Left + HorzScrollBar.ScrollPos) - (Self.Width diV 2);
    OutputDebugString(PChar(Format('Pos. actual= %d     Pos. Nueva=%d',
      [HorzScrollBar.ScrollPos, newPos])));

    // Asignar
    HorzScrollBar.Position := newPos;

  end;
end;



//function TPanelMiniaturas.GetPanelesCount(): integer;
//begin
//  Result := FPaneles.Count;
//end;


procedure TPanelMiniaturas.SetAnimated(const Value: boolean);
begin
  AnimTimer.Enabled := Value;
end;

procedure TPanelMiniaturas.DoAnimation(Sender: TObject);
begin

  // desactivado?
  if not (AnimTimer.Enabled) then begin
    Exit;
  end;

  // No hay elementos?
  if (Count = 0) then begin
    Exit;
  end;

  // Animar
  if (FItemIndex = -1) then begin
    ItemIndex := 0;
  end
  else if (FItemIndex >= (Count - 1)) then begin
    ItemIndex := 0;
  end
  else begin
    ItemIndex := ItemIndex + 1;
  end;

end;

procedure TPanelMiniaturas.SetAnimateInterval(const Value: cardinal);
begin
  AnimTimer.Interval := Value;
end;

function TPanelMiniaturas.GetAnimated(): boolean;
begin
  Result := AnimTimer.Enabled;
end;

function TPanelMiniaturas.GetAnimateInterval: cardinal;
begin
  Result := AnimTimer.Interval;
end;

end.

